<?php

namespace Intergo\Helpers;

use Intergo\Helpers\Calculators\SMSCounter;
use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumber;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberToCarrierMapper;
use libphonenumber\PhoneNumberUtil;
use PhoneNumberVO;

class PhoneHelper
{
    const MOBILE_FORMAT = 1;
    const FIXED_LINE_OR_MOBILE = 2;
    const TOLL_FREE = 3;
    const UNKNOWN = 10;

    /**
     * @param $phoneNumber
     * @param bool $sanitize
     * @return string
     */
    public static function cleanUpPhoneNumber($phoneNumber, bool $sanitize = false)
    {
        $hasPlusSign = self::startsWithPlusSign($phoneNumber);
        $phoneNumber = self::removeWhitespacesAndNonDigitCharacters($phoneNumber);
        if ($sanitize) {
            $phoneNumber = self::sanitizeToGSM($phoneNumber);
        }

        if (!$hasPlusSign) {
            if ($result = self::checkAndReplaceLeadingZerosWithPlus($phoneNumber)) {
                return $result;
            }
            // If it doesn't start with + sign and neither 00 then prepend + sign
            return '+' . $phoneNumber;
        }

        if ($result = self::checkAndRemoveZerosAfterPlus($phoneNumber)) {
            return $result;
        }
        return $phoneNumber;
    }


    /**
     * Prepend Plus sign on number if plus sign not exists already
     *
     * @param string $phoneNumber
     *
     * @return string
     */
    public static function prependPlusSign(string $phoneNumber)
    {
        return self::startsWithPlusSign($phoneNumber) ? $phoneNumber : '+' . $phoneNumber;
    }

    /**
     * Mask phone number.
     *
     * @param string $number
     * @return string
     */
    public static function maskPhoneNumber(string $number)
    {
        return substr($number, 0, 4) . '******' . substr($number, -2);
    }

    /**
     * @param $number
     * @return bool
     */
    public static function startsWithPlusSign($number)
    {
        return substr($number, 0, 1) == '+';
    }

    /**
     * @param $number
     * @return string
     */
    public static function removeWhitespacesAndNonDigitCharacters($number)
    {
        return preg_replace("/\s+[^0-9]/", "", $number);
    }

    /**
     * If it starts with 00 then replace with + sign
     *
     * @param $number
     * @return string|false
     */
    public static function checkAndReplaceLeadingZerosWithPlus($number)
    {
        if (preg_match("~^00\d+$~", $number)) {
            return preg_replace('/^00?/', "+", $number);
        }
        return false;
    }

    /**
     * If it starts with +00 (following any number of 0s) remove 0s and leave only + sign
     *
     * @param $number
     * @return string|false
     */
    public static function checkAndRemoveZerosAfterPlus($number)
    {
        if (preg_match("~\+00\d+$~", $number)) {
            return preg_replace('/^\+00?/', "+", $number);
        }
        return false;
    }

    /**
     * @param $number
     * @return array|string|string[]
     */
    private static function sanitizeToGSM($number)
    {
        $smsCounter = new SMSCounter();
        $number = $smsCounter->sanitizeToGSM($number);
        $number = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x9F]/u', '', $number);
        $number = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F]/u', '', $number);
        $number = preg_replace('/[\x00-\x1F\x7F-\xA0\xAD]/u', '', $number);
        $number = preg_replace('/[^[:print:]]/', '', $number);
        return str_replace([' ', '-', '(', ')'], '', $number);
    }

    /**
     * @param string $phone
     * @param string|null $defaultPrefix
     * @return PhoneNumberVO|null
     */
    public static function validatePhoneNumber(string $phone, string $defaultPrefix = null)
    {
        $phoneWithPlusSign = PhoneHelper::cleanUpPhoneNumber($phone);
        $phoneNumber = self::validatePhoneNumberByGoogle($phoneWithPlusSign, null);
        if (empty($phoneNumber)) {
            $phoneNumber = self::validatePhoneNumberByGoogle(substr($phoneWithPlusSign, 1), $defaultPrefix);
        }
        return self::getResult($phoneNumber);
    }

    /**
     * Validate phone number using PhoneNumberUtil
     *
     * @param string $phone
     * @param string|null $defaultPrefix country code
     *
     * @return PhoneNumber|null
     */
    public static function validatePhoneNumberByGoogle($phone, ?string $defaultPrefix)
    {
        try {
            $phoneNumberUtil = PhoneNumberUtil::getInstance();
            $phoneNumber = $phoneNumberUtil->parse($phone, $defaultPrefix);
            $phoneNumberType = $phoneNumberUtil->getNumberType($phoneNumber);
            if (self::isValidMobileFormat($phoneNumberType)) {
                return $phoneNumber;
            }
        } catch (NumberParseException $exception) {
        }
        return null;
    }

    /**
     * @param PhoneNumber|null $phoneNumber
     * @return PhoneNumberVO|null
     */
    public static function getResult(?PhoneNumber $phoneNumber)
    {
        if (is_null($phoneNumber)) {
            return null;
        }
        $phoneNumberUtil = PhoneNumberUtil::getInstance();
        $phone = $phoneNumberUtil->format($phoneNumber, PhoneNumberFormat::E164);
        $networkName = PhoneNumberToCarrierMapper::getInstance()->getNameForNumber($phoneNumber, "en");

        $phoneNumberVO = new PhoneNumberVO();
        $phoneNumberVO->setValid(true);
        $phoneNumberVO->setCountryCode($phoneNumber->getCountryCode());
        $phoneNumberVO->setNetworkName($networkName);
        $phoneNumberVO->setPhone(self::prependPlusSign($phone));
        $phoneNumberVO->setNumber($phoneNumber);
        $phoneNumberVO->setA2Code($phoneNumberUtil->getRegionCodeForNumber($phoneNumber));
        return $phoneNumberVO;
    }

    /**
     * @param $phoneNumberType
     * @return bool
     */
    public static function isValidMobileFormat($phoneNumberType)
    {
        if ($phoneNumberType == self::UNKNOWN || !in_array($phoneNumberType, self::getValidNumbers(), true)) {
            return false;
        }
        return true;
    }

    /**
     * @return int[]
     */
    private static function getValidNumbers(): array
    {
        return [self::FIXED_LINE_OR_MOBILE, self::MOBILE_FORMAT, self::TOLL_FREE];
    }
}
